#include "dispbios.h"
#include "filebios.h"

#define VRAMSIZE 0x28800
#define VRAMSIZE_CG20 1024
//----------------------------------------------------------------------------- std
#define RAND_MAX 32767

#define KEY_CTRL_FORMAT    30101
unsigned char ColorIndexDialog0( unsigned char initial_index );

int strlenOp( char *buffer ) ;
int strncmp(const char* s1, const char* s2, size_t n);
int sprintf(char *str,const char *fmt,...);
void qsort(void *aa, size_t n, size_t es, int (*cmp)(const void *, const void *));
void *memcpy2(void *buf1, const void *buf2, size_t n);

int OpcodeToStr2( int opcode, unsigned char *string ) ; // 
//----------------------------------------------------------------------------- math
double atof( char* buf ) ;

double fmod(double x,double y);
double fabs(double x);
double floor( double x ) ;
double sinh( double x );
double cosh( double x );
double tanh( double x );
double acos(double);
double asin(double);
double atan(double);
double atan2(double,double);
double cos(double);
double sin(double);
double tan(double);
double cosh(double);
double tanh(double);
double sinh(double);
double exp(double);
double log(double);
double log10(double);
double pow(double,double);
double sqrt(double);

//----------------------------------------------------------------------------- Bdisp
/*
void Bdisp_AllClr_DD(void);
void Bdisp_AllClr_VRAM(void);
void Bdisp_AllClr_DDVRAM(void);
void Bdisp_AreaClr_DD(const DISPBOX *pArea);
void Bdisp_AreaClr_VRAM(const DISPBOX *pArea);
void Bdisp_AreaClr_DDVRAM(const DISPBOX *pArea);
void Bdisp_AreaReverseVRAM(int x1, int y1, int x2, int y2);
void Bdisp_GetDisp_DD(unsigned char *pData);
void Bdisp_GetDisp_VRAM(unsigned char *pData);
void Bdisp_PutDisp_DD(void);
void Bdisp_PutDispArea_DD(const DISPBOX *PutDispArea);
void Bdisp_SetPoint_DD(int x, int y, unsigned char point);
void Bdisp_SetPoint_VRAM(int x, int y, unsigned char point);
void Bdisp_SetPoint_DDVRAM(int x, int y, unsigned char point);
int  Bdisp_GetPoint_VRAM(int x, int y);
void Bdisp_WriteGraph_DD(const DISPGRAPH *WriteGraph);
void Bdisp_WriteGraph_VRAM(const DISPGRAPH *WriteGraph);
void Bdisp_WriteGraph_DDVRAM(const DISPGRAPH *WriteGraph);
void Bdisp_ReadArea_DD(const DISPBOX *ReadArea, unsigned char *ReadData);
void Bdisp_ReadArea_VRAM(const DISPBOX *ReadArea, unsigned char *ReadData);
void Bdisp_DrawLineVRAM(int x1, int y1, int x2, int y2);
void Bdisp_ClearLineVRAM(int x1, int y1, int x2, int y2);

void locate(int x, int y);
void Print(const unsigned char *str);
void PrintRev(const unsigned char *str);
void PrintC(const unsigned char *c);
void PrintRevC(const unsigned char *str);
void PrintLine(const unsigned char *str, int max);
void PrintRLine(const unsigned char *str, int max);
void PrintXY(int x, int y, const unsigned char *str, int type);
int PrintMini(int x, int y, const unsigned char *str, int type);
void SaveDisp(unsigned char num);
void RestoreDisp(unsigned char num);
void PopUpWin(int n);

int Bfile_OpenFile(const FONTCHARACTER *filename, int mode);
int Bfile_OpenMainMemory(const unsigned char *name);
int Bfile_ReadFile(int HANDLE, void *buf, int size, int readpos);
int Bfile_WriteFile(int HANDLE, const void *buf, int size);
int Bfile_SeekFile(int HANDLE, int pos);
int Bfile_CloseFile(int HANDLE);
int Bfile_GetMediaFree(enum DEVICE_TYPE devicetype, int *freebytes);
int Bfile_GetFileSize(int HANDLE);
int Bfile_CreateFile(const FONTCHARACTER *filename, int size);
int Bfile_CreateDirectory(const FONTCHARACTER *pathname);
int Bfile_CreateMainMemory(const unsigned char *name);
int Bfile_RenameMainMemory(const unsigned char *oldname, const unsigned char *newname);
int Bfile_DeleteFile(const FONTCHARACTER *filename);
int Bfile_DeleteDirectory(const FONTCHARACTER *pathname);
int Bfile_DeleteMainMemory(const unsigned char *name);
int Bfile_FindFirst(const FONTCHARACTER *pathname, int *FindHandle, FONTCHARACTER *foundfile, FILE_INFO *fileinfo);
int Bfile_FindNext(int FindHandle, FONTCHARACTER *foundfile, FILE_INFO *fileinfo);
int Bfile_FindClose(int FindHandle);

void Bkey_Set_RepeatTime(long FirstCount, long NextCount);
void Bkey_Get_RepeatTime(long *FirstCount, long *NextCount);
void Bkey_Set_RepeatTime_Default(void);
*/

#define SAVEDISP_PAGE1              1
void SaveDisp( int n );
void RestoreDisp(int n);

void *GetVRAMAddress(void); // Return a pointer to the system's video memory.
void DD_SetContrast( int n ) ;
void Cursor_SetPosition( int x, int y );
int PutKey( unsigned int *key, int mode );
int OpCodeToStr( const short opcode, const char*result );
int Calljmp( int r4, int r5, int r6, int r7, int adrs );
int SysCalljmp( int r4, int r5, int r6, int r7, int No);

int PrintMinix1( int X, int Y, unsigned char*buf, int mode, unsigned short  color, unsigned short  backcolor ) ;
int PrintMinix3( int x, int y, unsigned char*buf, int mode ) ;
void Bdisp_AllClr_DDVRAM();

void SaveDisp( int n );
void RestoreDisp(int n);
void Bdisp_AllClr_DDVRAM();
void Bdisp_AreaReverseVRAM( int x1, int y1, int x2, int y2 ) ;
void Bdisp_AreaReverseVRAMx3( int x1, int y1, int x2, int y2 ) ;
void Bdisp_SetPoint_DDVRAM(int x, int y, unsigned char point);
void Bdisp_GetDisp_VRAM(unsigned char *pData);
void Bdisp_WriteGraph_RAM_fx(const DISPGRAPH *WriteGraph, unsigned char* ram );	// fx {128,64} mode
void Bdisp_WriteGraph_VRAM_fx(const DISPGRAPH *WriteGraph);	// fx {128,64} VRAM
void Bdisp_WriteGraph_VRAM_fx_xy1(const DISPGRAPH *WriteGraph);	// fx {384,192} VRAM
void Bdisp_WriteGraph_VRAM(const DISPGRAPH *WriteGraph);
void Bdisp_WriteGraph_DD(const DISPGRAPH *WriteGraph);
void Bdisp_ReadArea_VRAM_CG(const DISPBOX *ReadArea, unsigned char *ReadData);	// CG
void Bdisp_ReadArea_RAM_fx(const DISPBOX *ReadArea, unsigned char *ReadData, unsigned char* ram );	// fx {128,64} mode
void Bdisp_ReadArea_VRAM_fx(const DISPBOX *ReadArea, unsigned char *ReadData);	// fx
void Bdisp_ReadArea_VRAM(const DISPBOX *ReadArea, unsigned char *ReadData);	// CG or fx
void Bdisp_DrawLineVRAM(int x1, int y1, int x2, int y2);
void Bdisp_ClearLineVRAM(int x1, int y1, int x2, int y2);

void PopUpWin( int n );

#define locate locate_OS

//int Cursor_SetFlashOn( unsigned char cursor_type );
void Cursor_SetFlashMode( int mode ); 			// cursor flashing 
unsigned int Cursor_GetSettings(unsigned int *settingsarray); 
int Cursor_GetFlashStyle();

void SetStatusMessage( char*msg, char short_color );
void StatusArea( char*msg, char short_color );
void StatusArea_sub();
void StatusArea_Edit( char *name, int num);
void StatusArea_Run_sub( char *str );
void StatusArea_Run();

int GetColorIndex( void ) ;
int GetColor2Index( int colorRGB ) ;

//#define	SaveDisp(SAVEDISP_PAGE1)	memcpy(vbuf,(char*)GetVRAMAddress(),VRAMSIZE)
//#define	RestoreDisp(SAVEDISP_PAGE1)	memcpy((char*)GetVRAMAddress(),vbuf,VRAMSIZE)

//	memcpy(vbuf,(char*)GetVRAMAddress(),VRAMSIZE);	//	SaveDisp(SAVEDISP_PAGE1);
//	memcpy((char*)GetVRAMAddress(),vbuf,VRAMSIZE);	//	RestoreDisp(SAVEDISP_PAGE1);

//----------------------------------------------------------------------------- Bfile

int Bfile_OpenFile(const FONTCHARACTER *filename, int mode);
int Bfile_WriteFile(int HANDLE, const void *buf, int size);
int Bfile_GetMediaFree(enum DEVICE_TYPE devicetype, int *freebytes);
int Bfile_GetFileSize(int HANDLE);
int Bfile_CreateFile(const FONTCHARACTER *filename, int size);
int Bfile_CreateDirectory(const FONTCHARACTER *pathname);
int Bfile_OpenMainMemory(const unsigned char *name);
int Bfile_CreateMainMemory(const unsigned char *name);
int Bfile_DeleteFile(const FONTCHARACTER *filename);
int Bfile_DeleteDirectory(const FONTCHARACTER *pathname);
int Bfile_DeleteMainMemory(const unsigned char *name);

int Bfile_RenameEntry( const unsigned short *oldname , const unsigned short *newname );
int Bfile_GetFileInfo( const unsigned short *filename, FILE_INFO *fileinfo );

#define Bfile_CloseFile Bfile_CloseFile_OS
#define Bfile_SeekFile Bfile_SeekFile_OS
#define Bfile_ReadFile Bfile_ReadFile_OS
#define Bfile_WriteFile Bfile_WriteFile_OS
//-----------------------------------------------------------------------------

//----------------------------------------------------------------------------- Serial
int Serial_GetFreeTransmitSpace(); 
int Serial_BufferedTransmitOneByte( int c );
int Serial_ReadOneByte( char c );
int Serial_GetReceivedBytesAvailable();
void Serial_ClearTransmitBuffer();



//----------------------------------------------------------------------------- Timer
#define ID_USER_TIMER1          1
#define ID_USER_TIMER2          2
#define ID_USER_TIMER3          3
#define ID_USER_TIMER4          4
#define ID_USER_TIMER5          5

int SetTimer(int ID, int elapse, void (*hander)(void));
int KillTimer(int ID);

unsigned int SetGetkeyToMainFunctionReturnFlag( unsigned int enabled );

